var ws = new ActiveXObject("WScript.Shell"),
	fso = new ActiveXObject("Scripting.FileSystemObject"),
	installer = new ActiveXObject("WindowsInstaller.Installer");

function GetParentSourceDir() {
	return fso.GetFile(WScript.ScriptFullName).ParentFolder;
}

function WrapInQuote(str) {
	return "\"" + str + "\"";
}

function GetProductGUID(expectedName, expectedVersion) {
	// NAME - RegExp, required; VERSION - optional, format: "1.2.3456"
	var arVersion = [];
	if (expectedVersion) {
		arVersion = expectedVersion.split(".");
	}
	for(var product = new Enumerator(installer.Products); !product.atEnd(); product.moveNext())
	{
		var guid = product.item(),
			name = installer.ProductInfo(guid, "ProductName");
		if (expectedName.test(name))
		{
			if (!expectedVersion || (arVersion.length < 1)) {
				return guid;
			}
			var version = installer.ProductInfo(guid, "Version"),
				major = Math.floor(version >> 24);
			if (major != arVersion[0]) {
				return null;
			}

			if (arVersion.length < 2) {
				return guid;
			}
			var minor = Math.floor(version >> 16) - (major << 8);
			if (minor != arVersion[1]) {
				return null;
			}

			if (arVersion.length < 3) {
				return guid;
			}
			var revision = version % 65536;
			return (revision == arVersion[2]) ? guid : null;
		}
	}
	return null;
}

function ProductInstalled(expectedName, expectedVersion) {
	return (null !== GetProductGUID(expectedName, expectedVersion));
}

function CompareVersions(expectedVersion, actualVersion) {
	if (!expectedVersion) {
		return (null !== actualVersion);
	}
	if (!actualVersion) {
		return false;
	}
	var arVersion = actualVersion.split('.'),
		arExpectedVersion = expectedVersion.split('.');
	for(var i = 0, i1 = arVersion.length, i2 = arExpectedVersion.length; (i < i1) && (i < i2); ++i) {
		var diff = Number(arVersion[i]) - Number(arExpectedVersion[i]);
		if (0 !== diff) {
			return (diff > 0);
		}
	}
	return true;
}

function MSI31() {
	this.IsInstalled = function() {
		var expectedVersion = "3.1.4000.1823";
		return CompareVersions(expectedVersion, installer.Version);
	};
}

function CryptoPro() {
	this.IsInstalled = function() {
		return ProductInstalled(new RegExp("^(|CryptoPro)", "i"));
	};
}

function CryptoPro30() {
	this.IsInstalled = function() {
		return ProductInstalled(new RegExp("^(|CryptoPro)\\s+CSP", "i"), "3.0");
	};
}

function CryptoPro36() {
	this.IsInstalled = function() {
		return ProductInstalled(new RegExp("^(|CryptoPro)\\s+CSP", "i"), "3.6");
	};
}

function RuTokenDriver() {
	this.IsInstalled = function() {
		return ProductInstalled(new RegExp("^(Rutoken|).+(Driver|)", "i"));
	};
}

function RuTokenSupport() {
	this.IsInstalled = function() {
		return ProductInstalled(new RegExp("^(Rutoken)?\\s*(support\\s+module|\\s+)\\s*(Rutoken|)?", "i"));
	};
}

function RebootWindows() {
	var wmiService = GetObject("winmgmts:{impersonationLevel=impersonate,(Shutdown)}//./root/cimv2"),
		colOS = wmiService.ExecQuery("SELECT * FROM Win32_OperatingSystem WHERE Primary=true");
	for (var e = new Enumerator(colOS); !e.atEnd(); e.moveNext()) {
		e.item().Win32Shutdown(2); // 2==reboot; 6==force reboot
	}
}

function CARunSetup_MSI31() {
	var sourceDir = fso.BuildPath(GetParentSourceDir(), "..\\MSI3.1\\");
	ws.Run(WrapInQuote(sourceDir + "WindowsInstaller3.1.exe") + " /passive /norestart",1,1);
	ws.Popup("  Microsoft Windows Installer 3.1   .     .  'OK',   .", 0, "", 64);
	RebootWindows();
}

function CARunSetup_RutokenDriver() {
	var sourceDir = fso.BuildPath(GetParentSourceDir(), "\\"),
		arch = ws.ExpandEnvironmentStrings("%PROCESSOR_ARCHITECTURE%"),
		rtFileName = (arch.indexOf("64")>=0) ? "rtDrivers.x64.exe" : "rtDrivers.exe";
	ws.Run(WrapInQuote(sourceDir + rtFileName) + " /S /V" + WrapInQuote("/passive /norestart"),1,1);
}

function CARunSetup_RutokenSupport() {
	var parentSourceDir = GetParentSourceDir(),
		sourceDir = fso.BuildPath(parentSourceDir, "   \\"),
		csp30 = new CryptoPro30().IsInstalled(),
		csp36 = new CryptoPro36().IsInstalled(),
		rtSupFileName;
	if(csp30) {
		var parent2 = fso.GetFolder(parentSourceDir).ParentFolder,
			pcsc = fso.BuildPath(parent2, " 3.0\\reader\\pcsc.msp");
		ws.Run("msiexec.exe /update \"" + pcsc + "\" /qr /norestart",1,1);
	}
	if (csp30 || csp36) {
		var arch = ws.ExpandEnvironmentStrings("%PROCESSOR_ARCHITECTURE%");
		rtSupFileName = (arch.indexOf("64")>=0) ? "rtSup_for_CryptoPro_3.x_x64.exe" : "rtSup_for_CryptoPro_3.x.exe";
	} else {
		rtSupFileName = "rtSup_for_CryptoPro_2.0.exe";
	}
	ws.Run(WrapInQuote(sourceDir + rtSupFileName) + " /S /V" + WrapInQuote("/passive /norestart"),1,1);	
}

var somethingInstalled = false;

if (!(new MSI31().IsInstalled())) {
	CARunSetup_MSI31();
	somethingInstalled = true;
}
if ((new CryptoPro().IsInstalled()) && !(new RuTokenSupport().IsInstalled())) {
	CARunSetup_RutokenSupport();
	somethingInstalled = true;
}
if (!(new RuTokenDriver().IsInstalled())) {
	CARunSetup_RutokenDriver();
	somethingInstalled = true;
}
if (somethingInstalled) {
	ws.Popup("   .\r\n     .", 0, " ", 64);
} else {
	ws.Popup("   .", 0, "  ", 64);
}
